<?php

namespace Pinpoint\BlankProduct\Observer\Order;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer;

/**
 * Class Cancel
 * @package Pinpoint\BlankProduct\Observer\Order
 */
class Cancel implements ObserverInterface
{
    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $productFactory;

    /**
     * @var \Magento\CatalogInventory\Model\Stock\Item
     */
    protected $stockItem;

    /**
     * @var \Magento\Quote\Model\Quote\Item
     */
    protected $quoteItem;

    /**
     * Cancel constructor.
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\CatalogInventory\Model\Stock\Item $stockItem
     * @param \Magento\Quote\Model\Quote\Item $quoteItem
     */
    public function __construct(
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\CatalogInventory\Model\Stock\Item $stockItem,
        \Magento\Quote\Model\Quote\Item $quoteItem
    ) {
        $this->productFactory = $productFactory;
        $this->stockItem = $stockItem;
        $this->quoteItem = $quoteItem;
    }

    /**
     * Execute Observer
     *
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        /** @var \Magento\Sales\Model\Order $order */
        $order = $observer->getData('order');

        foreach ($order->getItems() as $orderItem) {
            $quoteItem = $this->quoteItem->load($orderItem->getQuoteItemId());
            if ($quoteItem->hasData('blank_products') && !is_null($quoteItem->getData('blank_products'))) {
                $cancelledQuantity = $orderItem->getQtyCanceled();

                $blankTotal = 0;
                foreach (json_decode($quoteItem->getData('blank_products'), true) as $product) {
                    $id = array_keys($product)[0];
                    $qty = $product[$id];

                    $productStockItem = $this->stockItem->load($id, 'product_id');
                    $productStockItem->setQty($productStockItem->getQty() + (int) $qty);

                    if ($productStockItem->getQty()) {
                        $productStockItem->setIsInStock(true);
                    }

                    $productStockItem->save();
                    $cancelledQuantity -= (int) $qty;
                    $blankTotal += (int) $qty;
                }

                $productStockItem = $this->stockItem->load($orderItem->getProductId(), 'product_id');
                $productStockItem->setQty($productStockItem->getQty() - $orderItem->getQtyCanceled());

                if (abs($cancelledQuantity)) {
                    if ($cancelledQuantity > 0) {
                        $productStockItem->setQty($productStockItem->getQty() + $cancelledQuantity);

                        if ($productStockItem->getQty()) {
                            $productStockItem->setIsInStock(true);
                        }
                    } else {
                        $productStockItem->setQty(0)->setIsInStock(false);
                    }

                    $productStockItem->save();
                }
            }
        }
    }
}
