<?php
namespace Pinpoint\BlankProduct\Plugin\Model;

use \Magento\Framework\Registry;
use \Magento\Backend\Model\Auth\Session as AuthSession;

/**
 * Class StockRegistryStoragePlugin
 * @package Pinpoint\BlankProduct\Plugin\Model
 */
class StockRegistryStoragePlugin
{
    /**
     * @var Registry
     */
    protected $_coreRegistry;

    /**
     * @var AuthSession
     */
    protected $_authSession;

    /**
     * @var \Pinpoint\BlankProduct\Helper\Quantity
     */
    protected $quantityHelper;

    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $request;

    /**
     * StockRegistryStoragePlugin constructor.
     * @param Registry $coreRegistry
     * @param AuthSession $authSession
     * @param \Pinpoint\BlankProduct\Helper\Quantity $quantityHelper
     * @param \Magento\Framework\App\Request\Http $request
     */
    public function __construct(
        Registry $coreRegistry,
        AuthSession $authSession,
        \Pinpoint\BlankProduct\Helper\Quantity $quantityHelper,
        \Magento\Framework\App\Request\Http $request
    ) {
        $this->_coreRegistry = $coreRegistry;
        $this->_authSession = $authSession;
        $this->quantityHelper = $quantityHelper;
        $this->request = $request;
    }

    /**
     * @param \Magento\CatalogInventory\Model\StockRegistryStorage $subject
     * @param $productId
     * @param $scopeId
     * @param \Magento\CatalogInventory\Api\Data\StockItemInterface $value
     */
    public function beforeSetStockItem(
        \Magento\CatalogInventory\Model\StockRegistryStorage $subject,
        $productId,
        $scopeId,
        \Magento\CatalogInventory\Api\Data\StockItemInterface $value
    ) {
        if(
            !$this->_authSession->isLoggedIn() && !is_null($this->request->getControllerModule())
            || $this->_authSession->isLoggedIn() && $this->request->getControllerModule() != "Magento_Catalog" && $this->request->getActionName() != "save"
        ) {
            if ($this->_coreRegistry->registry('stockitem_lookup') !== 'value') {
                $this->_coreRegistry->register('stockitem_lookup', 'value');
                $stockItem = $this->quantityHelper->getBlankProductStockItem($productId, true);
                if($stockItem) {
                    $value['qty'] += $stockItem->getQty();
                }
                $this->_coreRegistry->unregister('stockitem_lookup');
            }
        }
    }
}
